/**
 * 
 */
package edx;

import java.util.List;
import java.util.Random;

import org.jboss.logging.Logger;

import freemarker.template.TemplateMethodModel;
import freemarker.template.TemplateModelException;

/**
 * @author Edward Pilipczuk
 *
 */
public class IdGen implements TemplateMethodModel {

	public final String ID;
	private int count = 0;
	private Logger log = Logger.getLogger(this.getClass().getSimpleName());
	
	public IdGen() {
		super();
		ID = sgen();
		log.debug("Instantiated "+ID);
	}

	private synchronized int getNext() {
		count++;
		return count;
	}
	
	
	/**
	 * List contains operation name to be executed:
	 * "BATCH" - returns the Batch ID value
	 * "NEXT" - increments count and returns its value (autoincremented)
	 * "COUNT" - returns actual count value 
	 * "ID" - return the whole ID as Sring in a form of "COUNT'@'BATCH"    
	 */
	@SuppressWarnings("unchecked")
	public Object exec(List arg0) throws TemplateModelException {
		
		if( arg0.isEmpty() )
			throw new TemplateModelException("Submitted List cannot be empty!");
		else {
			if( !(arg0.get(0) instanceof String) ) 
				throw new TemplateModelException("First argument from the list is "+arg0.get(0).getClass().getSimpleName()+", must be of String type!");
			String cmd = (String) arg0.get(0);
			if( cmd.equalsIgnoreCase("BATCH") ) 
				return ID;
			else if( cmd.equalsIgnoreCase("COUNT") ) 
				return count;
			else if( cmd.equalsIgnoreCase("NEXT") ) 
				return getNext();
			else if( cmd.equalsIgnoreCase("ID") ) 
				return getIdAsString();
			else
				throw new TemplateModelException("Unsupported command: '"+cmd+"'");
		}
		
	}

	/**
	 * Return the Next value of ID as String Value
	 * @return
	 */
	private String getIdAsString() {
		return count+"@"+ID;
	}
	
	/**
	 * Generate HashCode for the IdGen bean instance 
	 * @return radom hex integer as string
	 */
	private String sgen() {
		// generate random value
		Random r = new Random( System.currentTimeMillis() );
		return Integer.toHexString(r.nextInt());
	}
	
}
